from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('',
        url(r'^channel\.html$', 'facebook_connect.views.channel_url', name='channel_url'),
        url(r'^facebook_connect/$', 'facebook_connect.views.facebook_connect', name='facebook_connect'),
    )




import datetime
import hashlib
import hmac
import json
import base64

from facebook_python_sdk import facebook

from django.conf import settings
from django.contrib import auth
from django.contrib.auth.models import User
from django.http import HttpResponse
from django.shortcuts import render_to_response
    
from facebook_connect.models import FacebookUser

from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST

@require_POST
@csrf_exempt
def facebook_connect(request):
    """
    Create or connect a user via facebook connect.
    """
    try:
        access_token = request.POST["access_token"]
        user_id      = request.POST["id"]
        request_sig = request.POST["sig"]

        sig, expected_sig = get_sig_and_expected_sig(request_sig, settings.FACEBOOK_APP_SECRET)

        if sig == expected_sig:
            try:
                f_user = FacebookUser.objects.get(facebook_id=user_id)

            except FacebookUser.DoesNotExist:

                # Facebook api to get the user profile
                profile = facebook.GraphAPI(access_token).get_object('me')

                first_name  = profile["first_name"]
                last_name   = profile["last_name"]

                # Create the user
                user = User()
                user.save()

                user.username = u"fbuser_%s" % user.id
                user.first_name = first_name
                user.last_name = last_name

                # Attempt to set the email
                try:
                    if "email" in settings.FACEBOOK_SCOPE:
                        email = profile["email"]
                        user.email = email
                except AttributeError:
                    pass

                # Create the facebook user
                f_user = FacebookUser(facebook_id=user_id,
                                      contrib_user=user)

                # Set a password
                temp = hashlib.new('sha1')
                temp.update(str(datetime.datetime.now()))
                password = temp.hexdigest()

                user.set_password(password)
                f_user.contrib_password = password

                # Save
                f_user.save()
                user.save()
                        
            # Authenticate and login
            authenticated_user = auth.authenticate(username=f_user.contrib_user.username,
                                                   password=f_user.contrib_password)
            auth.login(request, authenticated_user)

        else:

            content = {
                "is_error" : True,
                "error_text" : "Error connecting facebook connect user %s " % (user_id)
            }

            return json_response(content, 200)
            
    except Exception, e:
        content = {
            "is_error" : True,
            "error_text" : "Error in ajax call, exception: %s" % e
        }

        return json_response(content, 200)
            

    content = {
        "is_error" : False,
        "error_text" : None
    }

    return json_response(content, 200)

def channel_url(request):
    """
    The channel file contains a script to the javascript sdk.
    This also prevents extra hits to the site (?fb_xd_fragment).
    """
    return render_to_response('channel.html')

def json_response(content, status=200):
    """
    Return a JSON response.
    """
    json_content = json.dumps(content)
    response = HttpResponse(json_content, status=status)
    return response

def base64_url_decode(inp):
    """
    Base64 decode.
    """
    padding_factor = (4 - len(inp) % 4) % 4
    inp += "="*padding_factor 
    return base64.b64decode(unicode(inp).translate(dict(zip(map(ord, u'-_'), u'+/'))))

def get_sig_and_expected_sig(signed_request, secret):
    """
    Returns the decoded sig and the expected sig.
    """
    parts = signed_request.split('.', 2)
    encoded_sig = parts[0]
    payload = parts[1]

    sig = base64_url_decode(encoded_sig)
    data = json.loads(base64_url_decode(payload))

    if data.get('algorithm').upper() != 'HMAC-SHA256':
        return None
    else:
        expected_sig = hmac.new(secret, msg=payload, digestmod=hashlib.sha256).digest()

    return sig, expected_sig



from django.contrib.auth.models import User
from django.db import models

from django.contrib import admin
 
class FacebookUser(models.Model):
	facebook_id = models.CharField(max_length=100, unique=True)
	
	# Keep track of the facebook user -> django user mapping
	contrib_user = models.OneToOneField(User)
	contrib_password = models.CharField(max_length=100)

	def __unicode__(self):
		return "%s %s" % (self.contrib_user.first_name, self.contrib_user.last_name)

class FacebookUserAdmin(admin.ModelAdmin):
	pass

admin.site.register(FacebookUser, FacebookUserAdmin)




from setuptools import setup, find_packages

setup(
    name='django-facebook-connect',
    version=__import__('facebook_connect').__version__,
    license="MIT",
    install_requires = [
        'Django',
    ],
    description='Add facebook connect authentication to your Django website.',
    long_description=open('README.md').read(),
    author='Noam Sutskever',
    author_email='noamsu@gmail.com',
    url='http://github.com/noamsu/django-facebook-connect',
    download_url='http://github.com/noamsu/django-facebook-connect/downloads',
    include_package_data=True,
    packages=['facebook_connect'],
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ]
)


from django import template
from django.conf import settings
from django.core.urlresolvers import reverse
    
register = template.Library()
    
class FacebookClientScriptNode(template.Node):
    def render(self, context):

        scope = getattr(settings, "FACEBOOK_SCOPE", "email")
        redirect = getattr(settings, "FACEBOOK_LOGIN_REDIRECT", "/")

        facebook_app_id = settings.FACEBOOK_APP_ID
        channel_url = reverse('channel_url')

        params = {
            "scope":scope,
            "facebook_app_id":facebook_app_id,
            "channel_url":channel_url,
            "redirect":redirect
        }

        return """

<div id="fb-root"></div>
<script type="text/javascript"> 
    
    function facebook_login(){ 
        FB.login(function(response){
            send_response_to_server(response)
        }, {scope:"%(scope)s"}
       )
    }

    function facebook_connect(){
        FB.getLoginStatus(function(response) {

            if (response.status == "connected") {
                send_response_to_server(response);
            } else {
                facebook_login();
            }   

        });
    }

    function send_response_to_server(response){

        var ajax_data = {"access_token" : response.authResponse.accessToken,
                         "sig"          : response.authResponse.signedRequest,
                         "id"           : response.authResponse.userID}

        $.ajax({
            type:"POST",
            url: "/facebook_connect/facebook_connect/",
            dataType: "json",
            data: ajax_data,
            success: function(msg){
                window.location = "%(redirect)s";
            }
        })
    }

    window.fbAsyncInit = function() {
        FB.init({
            appId: '%(facebook_app_id)s',
            status : true,
            cookie: true,
            xfbml : true,
            channelURL : '%(channel_url)s'
        })
    };
    (function(d){
        var js, id = 'facebook-jssdk', ref = d.getElementsByTagName('script')[0];
        if (d.getElementById(id)) {return;}
        js = d.createElement('script'); js.id = id; js.async = true;
        js.src = "//connect.facebook.net/en_US/all.js";
        ref.parentNode.insertBefore(js, ref);
    }(document));
</script>
""" % params

class FacebookLoginButtonNode(template.Node):
    def render(self, context): 
        return """

<input
    onclick="facebook_connect()"
    type="image"
    src="http://static.ak.fbcdn.net/images/fbconnect/login-buttons/connect_light_large_long.gif" alt="Facebook Connect" />
"""

def register_tag(tag_func):
    """
    Decorator for registering tags.
    """
    def new_tag_func(*args, **kwargs):
        return tag_func(*args, **kwargs)

    register.tag(tag_func)
    return new_tag_func

@register_tag
def facebook_script(parser, token): 
    return FacebookClientScriptNode()

@register_tag
def facebook_button(parser, token): 
    return FacebookLoginButtonNode()



#!/usr/bin/env python
#
# Copyright 2010 Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Python client library for the Facebook Platform.

This client library is designed to support the Graph API and the official
Facebook JavaScript SDK, which is the canonical way to implement
Facebook authentication. Read more about the Graph API at
http://developers.facebook.com/docs/api. You can download the Facebook
JavaScript SDK at http://github.com/facebook/connect-js/.

If your application is using Google AppEngine's webapp framework, your
usage of this module might look like this:

    user = facebook.get_user_from_cookie(self.request.cookies, key, secret)
    if user:
        graph = facebook.GraphAPI(user["access_token"])
        profile = graph.get_object("me")
        friends = graph.get_connections("me", "friends")

"""

import cgi
import hashlib
import time
import urllib

# Find a JSON parser
try:
    import json
    _parse_json = lambda s: json.loads(s)
except ImportError:
    try:
        import simplejson
        _parse_json = lambda s: simplejson.loads(s)
    except ImportError:
        # For Google AppEngine
        from django.utils import simplejson
        _parse_json = lambda s: simplejson.loads(s)


class GraphAPI(object):
    """A client for the Facebook Graph API.

    See http://developers.facebook.com/docs/api for complete documentation
    for the API.

    The Graph API is made up of the objects in Facebook (e.g., people, pages,
    events, photos) and the connections between them (e.g., friends,
    photo tags, and event RSVPs). This client provides access to those
    primitive types in a generic way. For example, given an OAuth access
    token, this will fetch the profile of the active user and the list
    of the user's friends:

       graph = facebook.GraphAPI(access_token)
       user = graph.get_object("me")
       friends = graph.get_connections(user["id"], "friends")

    You can see a list of all of the objects and connections supported
    by the API at http://developers.facebook.com/docs/reference/api/.

    You can obtain an access token via OAuth or by using the Facebook
    JavaScript SDK. See http://developers.facebook.com/docs/authentication/
    for details.

    If you are using the JavaScript SDK, you can use the
    get_user_from_cookie() method below to get the OAuth access token
    for the active user from the cookie saved by the SDK.
    """
    def __init__(self, access_token=None):
        self.access_token = access_token

    def get_object(self, id, **args):
        """Fetchs the given object from the graph."""
        return self.request(id, args)

    def get_objects(self, ids, **args):
        """Fetchs all of the given object from the graph.

        We return a map from ID to object. If any of the IDs are invalid,
        we raise an exception.
        """
        args["ids"] = ",".join(ids)
        return self.request("", args)

    def get_connections(self, id, connection_name, **args):
        """Fetchs the connections for given object."""
        return self.request(id + "/" + connection_name, args)

    def put_object(self, parent_object, connection_name, **data):
        """Writes the given object to the graph, connected to the given parent.

        For example,

            graph.put_object("me", "feed", message="Hello, world")

        writes "Hello, world" to the active user's wall. Likewise, this
        will comment on a the first post of the active user's feed:

            feed = graph.get_connections("me", "feed")
            post = feed["data"][0]
            graph.put_object(post["id"], "comments", message="First!")

        See http://developers.facebook.com/docs/api#publishing for all of
        the supported writeable objects.

        Most write operations require extended permissions. For example,
        publishing wall posts requires the "publish_stream" permission. See
        http://developers.facebook.com/docs/authentication/ for details about
        extended permissions.
        """
        assert self.access_token, "Write operations require an access token"
        return self.request(parent_object + "/" + connection_name, post_args=data)

    def put_wall_post(self, message, attachment={}, profile_id="me"):
        """Writes a wall post to the given profile's wall.

        We default to writing to the authenticated user's wall if no
        profile_id is specified.

        attachment adds a structured attachment to the status message being
        posted to the Wall. It should be a dictionary of the form:

            {"name": "Link name"
             "link": "http://www.example.com/",
             "caption": "{*actor*} posted a new review",
             "description": "This is a longer description of the attachment",
             "picture": "http://www.example.com/thumbnail.jpg"}

        """
        return self.put_object(profile_id, "feed", message=message, **attachment)

    def put_comment(self, object_id, message):
        """Writes the given comment on the given post."""
        return self.put_object(object_id, "comments", message=message)

    def put_like(self, object_id):
        """Likes the given post."""
        return self.put_object(object_id, "likes")

    def delete_object(self, id):
        """Deletes the object with the given ID from the graph."""
        self.request(id, post_args={"method": "delete"})

    def request(self, path, args=None, post_args=None):
        """Fetches the given path in the Graph API.

        We translate args to a valid query string. If post_args is given,
        we send a POST request to the given path with the given arguments.
        """
        if not args: args = {}
        if self.access_token:
            if post_args is not None:
                post_args["access_token"] = self.access_token
            else:
                args["access_token"] = self.access_token
        post_data = None if post_args is None else urllib.urlencode(post_args)
        file = urllib.urlopen("https://graph.facebook.com/" + path + "?" +
                              urllib.urlencode(args), post_data)
        try:
            response = _parse_json(file.read())
        finally:
            file.close()
        if response.get("error"):
            raise GraphAPIError(response["error"]["type"],
                                response["error"]["message"])
        return response


class GraphAPIError(Exception):
    def __init__(self, type, message):
        Exception.__init__(self, message)
        self.type = type


def get_user_from_cookie(cookies, app_id, app_secret):
    """Parses the cookie set by the official Facebook JavaScript SDK.

    cookies should be a dictionary-like object mapping cookie names to
    cookie values.

    If the user is logged in via Facebook, we return a dictionary with the
    keys "uid" and "access_token". The former is the user's Facebook ID,
    and the latter can be used to make authenticated requests to the Graph API.
    If the user is not logged in, we return None.

    Download the official Facebook JavaScript SDK at
    http://github.com/facebook/connect-js/. Read more about Facebook
    authentication at http://developers.facebook.com/docs/authentication/.
    """
    cookie = cookies.get("fbs_" + app_id, "")
    if not cookie: return None
    args = dict((k, v[-1]) for k, v in cgi.parse_qs(cookie.strip('"')).items())
    payload = "".join(k + "=" + args[k] for k in sorted(args.keys())
                      if k != "sig")
    sig = hashlib.md5(payload + app_secret).hexdigest()
    expires = int(args["expires"])
    if sig == args.get("sig") and (expires == 0 or time.time() < expires):
        return args
    else:
        return None